package com.nisum.poc.facade.client.impl;

import com.nisum.poc.facade.client.IEmployeeClient;
import com.nisum.poc.facade.response.EmployeeDetails;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.util.UriBuilder;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Repository
public class EmployeeClient implements IEmployeeClient {

    private WebClient.Builder employeeWebClient;

    @Autowired
    public EmployeeClient(@Qualifier("empWebClient") WebClient.Builder employeeWebClient) {
        this.employeeWebClient = employeeWebClient;
    }

    @Override
    public Flux<EmployeeDetails> fetchAllEmployee() {
        return employeeWebClient.build().get().uri("all").retrieve().bodyToFlux(EmployeeDetails.class);
    }

    @Override
    public Mono<EmployeeDetails> findEmployeeDetails(Long empId) {

        return employeeWebClient.build().get()
                .uri(uriBuilder -> uriBuilder.path("/v1/{empId}/details").build(empId))
                .retrieve().bodyToMono(EmployeeDetails.class);
    }
}
