package com.nisum.poc.facade.service.impl;

import com.nisum.poc.facade.client.IDepartmentClient;
import com.nisum.poc.facade.client.IEmployeeClient;
import com.nisum.poc.facade.client.IProjectClient;
import com.nisum.poc.facade.response.DepartmentDetails;
import com.nisum.poc.facade.response.EmployeeDetails;
import com.nisum.poc.facade.response.EmployeeFacadeResponse;
import com.nisum.poc.facade.response.ProjectDetails;
import com.nisum.poc.facade.service.IEmployeeService;
import org.modelmapper.ModelMapper;
import org.modelmapper.convention.MatchingStrategies;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class EmployeeService implements IEmployeeService {

    Logger logger = LoggerFactory.getLogger(EmployeeService.class);

    private final IEmployeeClient employeeClient;

    private final IDepartmentClient departmentClient;

    private final IProjectClient projectClient;

    @Autowired
    public EmployeeService (IEmployeeClient employeeClient, IDepartmentClient departmentClient, IProjectClient projectClient){
        this.employeeClient = employeeClient;
        this.departmentClient = departmentClient;
        this.projectClient = projectClient;

    }
    @Override
    public Flux<EmployeeDetails> fetchEmployeeDetails() {

        logger.info("Service Reached ");
        return employeeClient.fetchAllEmployee();
    }

    @Override
    public Mono<EmployeeFacadeResponse> fetchEmployeeById(Long empId) {

        logger.info(" Started fetchEmployeeById ");
        Mono<EmployeeDetails> monoEmployeeDetails = employeeClient.findEmployeeDetails(empId);
        return monoEmployeeDetails.zipWhen(employeeDetails -> findProjectAndDeptDetails(employeeDetails),
                (employeeDetails, facadeResponse) -> facadeResponse);

    }

    private Mono<EmployeeFacadeResponse> findProjectAndDeptDetails(EmployeeDetails employeeDetails){

        logger.info(" Started findProjectAndDeptDetails ");
        EmployeeFacadeResponse response = constructEmployeeFacadeResponse(employeeDetails);

        Long deptId = employeeDetails.getDeptId();
        Long projectId = employeeDetails.getProjectId();
        Mono<DepartmentDetails> departmentDetailsMono = departmentClient.fetchDepartmentById(deptId);
        Mono<ProjectDetails> projectDetailsMono = projectClient.findProjectDetails(projectId);

        return Mono.zip(departmentDetailsMono, projectDetailsMono).map( objects -> {

            response.setDepartmentDetails(objects.getT1());
            response.setProjectDetails(objects.getT2());

            return response;
        });
    }

    private Object getFacadeResponse(ProjectDetails project, EmployeeFacadeResponse response) {
        response.setProjectDetails(project);
        return response;
    }


    private Object getFacadeResponse(DepartmentDetails dept, EmployeeFacadeResponse response) {
        response.setDepartmentDetails(dept);
        return response;
    }
    private EmployeeFacadeResponse constructEmployeeFacadeResponse(EmployeeDetails employeeDetails) {

        EmployeeFacadeResponse employeeFacadeResponse;

        ModelMapper modelMapper = new ModelMapper();
        modelMapper.getConfiguration().setMatchingStrategy(MatchingStrategies.STRICT);
        employeeFacadeResponse = modelMapper.map(employeeDetails, EmployeeFacadeResponse.class);

        return employeeFacadeResponse;
    }
}
